using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace hammer
{
    public class ColorPicker : ComboBox
    {
        private static Dictionary<string, Color> allColors = CreateDictionary();

        private static Dictionary<string, Color> CreateDictionary()
        {
            Dictionary<string, Color> result = new Dictionary<string, Color>();
            result.Add("Black", Color.Black);
            result.Add("White", Color.White);
            result.Add("Blue", Color.Blue);
            result.Add("Red", Color.Red);
            result.Add("Maroon", Color.Maroon);
            result.Add("Green", Color.Green);
            result.Add("Lime", Color.Lime);
            result.Add("Silver", Color.Silver);
            result.Add("Gray", Color.Gray);
            result.Add("Navy", Color.Navy);
            result.Add("Yellow", Color.Yellow);
            result.Add("Olive", Color.Olive);
            result.Add("Cyan", Color.Cyan);
            result.Add("Teal", Color.Teal);
            result.Add("Purple", Color.Purple);
            result.Add("Magenta", Color.Magenta);
            return result;
        }

        private Color customColor = Color.White;

        public ColorPicker()
        {
            this.DropDownStyle = ComboBoxStyle.DropDownList;
            this.DrawMode = DrawMode.OwnerDrawFixed;
            foreach (string colorName in allColors.Keys)
                Items.Add(colorName);
            Items.Add("Custom");
        }

        [System.ComponentModel.Browsable(true)]
        [System.ComponentModel.Category("Appearance")]
        public Color SelectedColor
        {
            get
            {
                int idx = SelectedIndex;
                if (idx < 0)
                    return customColor;
                string colorName = Items[idx].ToString();
                if (colorName == "Custom")
                    return customColor;
                else
                    return allColors[colorName];
            }
            set
            {
                foreach (string colorName in allColors.Keys)
                    if (allColors[colorName] == value)
                    {
                        SelectedIndex = Items.IndexOf(colorName);
                        return;
                    }
                SelectedIndex = Items.IndexOf("Custom");
                CustomColor = value;
                Invalidate();
            }
        }

        [System.ComponentModel.Browsable(false)]
        [System.ComponentModel.DesignerSerializationVisibility(
            System.ComponentModel.DesignerSerializationVisibility.Hidden)]
        public Color CustomColor
        {
            get { return customColor; }
            set { customColor = value; }
        }

        protected override void OnDrawItem(DrawItemEventArgs e)
        {
            e.DrawBackground();
            if (e.Index != -1)
            {
                string itemText = Items[e.Index].ToString();
                Color itemColor = itemText == "Custom" ? customColor : allColors[itemText];

                Rectangle rectangle = new Rectangle(
                    3, e.Bounds.Top + 2, e.Bounds.Height, e.Bounds.Height - 4);
                using (SolidBrush brush = new SolidBrush(itemColor))
                {
                    e.Graphics.FillRectangle(brush, rectangle);
                    e.Graphics.DrawRectangle(Pens.Black, rectangle);
                }
                e.Graphics.DrawString(itemText, e.Font,
                    (e.State & DrawItemState.Selected) != 0 ? Brushes.White : Brushes.Black,
                    new RectangleF(e.Bounds.X + rectangle.Width + 4, e.Bounds.Y + 1,
                        e.Bounds.Width, e.Bounds.Height));
            }
            e.DrawFocusRectangle();
        }
    }
}
