using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using IntSight.Controls;
using IntSight.Controls.CodeModel;

namespace hammer
{
    public partial class SnippetBox : Form
    {
        private string searchStr = String.Empty;
        private ICodeSnippet[] snippets;
        private ICodeSnippetCallback callback;

        public static void Select(ICodeSnippet[] snippets,
            Control editor, int x, int y, ICodeSnippetCallback callback)
        {
            if (snippets == null || snippets.Length == 0)
                FindDialog.Beep();
            else
                new SnippetBox(snippets, editor, x, y, callback).Show();
        }

        private SnippetBox()
        {
            InitializeComponent();
        }

        private SnippetBox(ICodeSnippet[] snippets, Control editor, int x, int y,
            ICodeSnippetCallback callback)
        {
            InitializeComponent();
            Opacity = MainForm.GlobalOpacity;
            if (editor != null)
                Location = editor.PointToScreen(new Point(x, y));
            this.snippets = snippets;
            foreach (ICodeSnippet snippet in snippets)
            {
                string name = snippet.Name;
                if (name.Length > 8)
                    name += "\t";
                else
                    name += "\t\t";
                listBox.Items.Add(name + snippet.Description);
            }
            listBox.SelectedIndex = 0;
            if (listBox.Height > listBox.ItemHeight * snippets.Length)
                listBox.Height = listBox.ItemHeight * (snippets.Length + 1);
            this.ClientSize = listBox.Size;
            this.callback = callback;
        }

        private void listBox_DoubleClick(object sender, EventArgs e)
        {
            timer.Enabled = false;
            if (listBox.SelectedIndex != ListBox.NoMatches)
                callback.Expand(snippets[listBox.SelectedIndex], false);
            this.Close();
        }

        private void SnippetBox_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Enter:
                    if (listBox.SelectedIndex != ListBox.NoMatches)
                        callback.Expand(snippets[listBox.SelectedIndex], false);
                    break;
                case Keys.Escape:
                    break;
                default:
                    return;
            }
            timer.Enabled = false;
            this.Close();
            e.Handled = true;
        }

        private void SnippetBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar >= ' ')
                searchStr += e.KeyChar.ToString();
            else if (e.KeyChar == 8)
            {
                if (searchStr.Length > 0)
                    searchStr = searchStr.Remove(searchStr.Length - 1);
            }
            else
                return;
            if (searchStr.Length == 0)
                listBox.SelectedIndex = 0;
            else
            {
                int idx = listBox.FindString(searchStr, -1);
                if (idx != ListBox.NoMatches)
                    listBox.SelectedIndex = idx;
            }
            e.Handled = true;
            timer.Enabled = true;
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            timer.Enabled = false;
            searchStr = String.Empty;
        }

        private void SnippetBox_Deactivate(object sender, EventArgs e)
        {
            Close();
        }
    }
}